<?php

namespace Drupal\ui_patterns_settings\Plugin\UiPatterns\SettingType;

use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;
use Drupal\ui_patterns_settings\Plugin\ComplexSettingTypeBase;

/**
 * Complex setting type for links.
 *
 * @UiPatternsSettingType(
 *   id = "links",
 *   label = @Translation("Links")
 * )
 */
class LinksSettingType extends ComplexSettingTypeBase {

  /**
   * Normalize menu items.
   *
   * Don't inject URL object into patterns templates, use "title" as item
   * label and "url" as item target.
   *
   * @param array $items
   *   The items to convert.
   *
   * @return array
   */
  public static function normalize(array $items): array {
    foreach ($items as $index => &$item) {
      if (!is_array($item)) {
        unset($items[$index]);
        continue;
      }
      if (array_key_exists("text", $item)) {
        // Examples: links.html.twig, breadcrumb.html.twig, pager.html.twig,
        // views_mini_pager.html.twig.
        $item["title"] = $item["text"];
        unset($item["text"]);
      }
      if (!array_key_exists("title", $item)) {
        $item["title"] = $index;
      }
      if (array_key_exists("href", $item)) {
        // Examples: pager.html.twig, views_mini_pager.html.twig.
        $item["url"] = $item["href"];
        unset($item["href"]);
      }
      if (!isset($item["url"]) && isset($item["link"])) {
        // Example: links.html.twig.
        $item["url"] = $item["link"]["#url"];
        $item["url"]->setOptions($item["link"]["#options"]);
        unset($item["link"]);
      }
      $item = self::normalizeUrl($item);
      if (array_key_exists("below", $item)) {
        $item["below"] = self::normalize($item["below"]);
      }
    }
    $items = array_values($items);
    return $items;
  }

  /**
   * Normaize URL in an item.
   *
   * Useful for: menu.html.twig, links.html.twig.
   */
  private static function normalizeUrl(array $item): array {
    if (!array_key_exists("url", $item)) {
      return $item;
    }
    $url = $item["url"];
    if (!($url instanceof Url)) {
      return $item;
    }
    if ($url->isRouted() && ($url->getRouteName() === '<nolink>')) {
      unset($item["url"]);
    }
    elseif ($url->isRouted() && ($url->getRouteName() === '<button>')) {
      unset($item["url"]);
    }
    else {
      $item["url"] = $url->toString();
    }
    $options = $url->getOptions();
    if (isset($options["attributes"])) {
      $item["link_attributes"] = new Attribute($options["attributes"]);
    }
    return $item;
  }

}
